"""
Utility functions to get the length of a char and to get the hex colour from
an ansi code
"""
from __future__ import annotations
from typing import Optional

from pathlib import Path
from yaml import safe_load

THISDIR = str(Path(__file__).resolve().parent)


def rgbToHex(rgb: tuple[int, int, int]) -> str:
	""" convert rgb tuple to hex """
	return "#{0:02x}{1:02x}{2:02x}".format(rgb[0], rgb[1], rgb[2])


def ansiTrueToRgb(ansiTrue: str):
	""" convert ANSI truecolour to hex rgb """
	rgb = ansiTrue.replace("\033[", "").replace("38;2;", "").replace("48;2;",
	"").replace("m", "").split(";")
	return rgbToHex((int(rgb[0]), int(rgb[1]), int(rgb[2])))


def ansi256ToRGB(ansi256: str, theme: Optional[str]=None) -> str:
	"""  convert ANSI 256 to hex rgb """
	# 0-7, 8-15
	switch = int(
	ansi256.replace("\033[", "").replace("38;5;", "").replace("48;5;",
	"").replace("m", ""))
	ansi16Map = {
	0: "base01", 1: "base08", 2: "base0B", 3: "base09", 4: "base0D", 5: "base0E",
	6: "base0C", 7: "base06", 8: "base02", 9: "base12", 10: "base14",
	11: "base13", 12: "base16", 13: "base17", 14: "base15", 15: "base07"}
	if switch < 16:
		return ansi16ToRGB("\033[{}m".format(switch), ansi16Map, theme)
	# 6^3
	if switch < 232:
		switch -= 16
		red = switch // 36
		switch -= red * 36
		green = switch // 6
		switch -= green * 6
		blue = switch
		return rgbToHex((red * 51, green * 51, blue * 51))
	# 232-255
	switch -= 232
	return rgbToHex((switch * 11, switch * 11, switch * 11))


def ansi16ToRGB(ansi16: str, ansi16Map: Optional[dict[int, str]]=None, theme: Optional[str]=None):
	"""  convert ANSI 16 to hex rgb """
	cCode = int(ansi16.replace("\033[", "").replace("m", ""))
	if 39 < cCode < 48 or 99 < cCode < 108:
		cCode -= 10
	ansi16Map = ansi16Map if ansi16Map is not None else {
	30: "base01", 31: "base08", 32: "base0B", 33: "base09", 34: "base0D",
	35: "base0E", 36: "base0C", 37: "base06", 90: "base02", 91: "base12",
	92: "base14", 93: "base13", 94: "base16", 95: "base17", 96: "base15",
	97: "base07"}
	return "#" + safe_load(open(theme if theme is not None else THISDIR + "/onedark.yml"))[ansi16Map[cCode]]


def ansiColourToRGB(ansiColour: str, theme: Optional[str]=None):
	""" convert an ANSI colour to a hex colour

	Args:
		ansiColour (string): ANSI colour

	Returns:
		string: hex code

	"""
	# 38;5
	if ansiColour.startswith("\033[38;5"):
		return ansi256ToRGB(ansiColour, theme)
	# 48;5
	if ansiColour.startswith("\033[48;5"):
		return ansi256ToRGB(ansiColour, theme)
	# 38;2
	if ansiColour.startswith("\033[38;2"):
		return ansiTrueToRgb(ansiColour)
	# 48;2
	if ansiColour.startswith("\033[48;2"):
		return ansiTrueToRgb(ansiColour)
	# 30 - 37
	if ansiColour.startswith("\033[3"):
		return ansi16ToRGB(ansiColour, theme=theme)
	# 90 - 97
	if ansiColour.startswith("\033[9"):
		return ansi16ToRGB(ansiColour, theme=theme)
	# 40 - 47
	if ansiColour.startswith("\033[4"):
		return ansi16ToRGB(ansiColour, theme=theme)
	# 100 - 107
	if ansiColour.startswith("\033[10"):
		return ansi16ToRGB(ansiColour, theme=theme)
	# fail on fg colour
	return "#" + safe_load(open(theme if theme is not None else THISDIR + "/onedark.yml"))["base05"]


def findLen(string: list[str]):
	""" find the length of a string and take into account that emojis are double
	width """
	counter = 0
	for i in string:
		if ord(i) > 10000: # emoji is double width
			counter += 1
		counter += 1
	return counter



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ansitoimg']

package_data = \
{'': ['*'], 'ansitoimg': ['resources/*']}

install_requires = \
['MarkupSafe>=1.1.1,<3',
 'Pillow>=8.0.0,<10',
 'PyYAML>=5.3.1,<7',
 'pyppeteer>=0.2.2,<2',
 'svgwrite>=1.4,<3']

entry_points = \
{'console_scripts': ['ansitoimg = ansitoimg:cli']}

setup_kwargs = {
    'name': 'ansitoimg',
    'version': '2020.4.2',
    'description': 'Convert an ANSI string to an image. Great for adding terminal output into a readme.',
    'long_description': '[![Github top language](https://img.shields.io/github/languages/top/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](../../)\n[![Codacy grade](https://img.shields.io/codacy/grade/[proj].svg?style=for-the-badge)](https://www.codacy.com/gh/FHPythonUtils/AnsiToImg)\n[![Repository size](https://img.shields.io/github/repo-size/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](../../)\n[![Issues](https://img.shields.io/github/issues/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](../../issues)\n[![License](https://img.shields.io/github/license/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](/LICENSE.md)\n[![Commit activity](https://img.shields.io/github/commit-activity/m/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](../../commits/master)\n[![Last commit](https://img.shields.io/github/last-commit/FHPythonUtils/AnsiToImg.svg?style=for-the-badge)](../../commits/master)\n[![PyPI Downloads](https://img.shields.io/pypi/dm/ansitoimg.svg?style=for-the-badge)](https://pypi.org/project/ansitoimg/)\n[![PyPI Version](https://img.shields.io/pypi/v/ansitoimg.svg?style=for-the-badge)](https://pypi.org/project/ansitoimg/)\n\n<!-- omit in toc -->\n# AnsiToImg\n\n<img src="readme-assets/icons/name.png" alt="Project Icon" width="750">\n\nConvert an ANSI string to an image. Great for adding terminal output into a readme.\n\n- [Examples](#examples)\n\t- [SVG Image](#svg-image)\n\t- [Raster Image](#raster-image)\n\t- [SVGRaster Image](#svgraster-image)\n\t- [HTML/ HTMLRaster Image](#html-htmlraster-image)\n\t- [Windows Terminal](#windows-terminal)\n- [Choosing ansiToSVG, ansiToRaster, ansiToSVGRaster, ansiToHTML or ansiToHTMLRaster](#choosing-ansitosvg-ansitoraster-ansitosvgraster-ansitohtml-or-ansitohtmlraster)\n\t- [ansiToSVG](#ansitosvg)\n\t- [ansiToRaster](#ansitoraster)\n\t- [ansiToSVGRaster](#ansitosvgraster)\n\t- [ansiToHTML](#ansitohtml)\n\t- [ansiToHTMLRaster](#ansitohtmlraster)\n- [Changelog](#changelog)\n- [Install With PIP](#install-with-pip)\n- [Language information](#language-information)\n\t- [Built for](#built-for)\n- [Install Python on Windows](#install-python-on-windows)\n\t- [Chocolatey](#chocolatey)\n\t- [Download](#download)\n- [Install Python on Linux](#install-python-on-linux)\n\t- [Apt](#apt)\n- [How to run](#how-to-run)\n\t- [With VSCode](#with-vscode)\n\t- [From the Terminal](#from-the-terminal)\n- [How to update, build and publish](#how-to-update-build-and-publish)\n- [Community Files](#community-files)\n\t- [Licence](#licence)\n\t- [Changelog](#changelog-1)\n\t- [Code of Conduct](#code-of-conduct)\n\t- [Contributing](#contributing)\n\t- [Security](#security)\n\t- [Support](#support)\n\n## Examples\n\nHere is an example of some code and the images it produces:\n\nFunctions accept the following arguments:\n- ansiText - text to process\n- fileName - name of the file to write to\n- theme - a base24 theme. Defaults to atom one dark\n\n```python\nimport sys\nimport os\nfrom pathlib import Path\nimport platform\nimport ctypes\nfrom catimage.catimage import generateHDColour\n\nTHISDIR = str(Path(__file__).resolve().parent)\nsys.path.insert(0, os.path.dirname(THISDIR))\nfrom ansitoimg.render import ansiToSVG, ansiToRaster, ansiToSVGRaster, ansiToHTML, ansiToHTMLRaster\n\n\nif platform.system() == "Windows":\n\tkernel32 = ctypes.windll.kernel32\n\tkernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)\n\n# Define ANSI text\nexample = "👋\\033[32mHello\\033[0m, \\033[34mWorld\\033[0m🌏\\033[31m!\\033[0m\\n\\033[41m👋\\033[0m\\033[43m🦄\\033[0m\\033[42m🐘\\033[0m\\033[3m\\033[9m13\\033[0m\\033[1m3\\033[0m\\033[4m7\\033[0m\\033[46m🍄\\033[0m\\033[44m🎃\\033[0m\\033[45m🐦\\033[0m"\nexample2 = "hello\\nworld\\n\\033[42m\\033[31mwe meet again\\033[0m\\nABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz😁😂🤣😃😄😅😆😉😊😋😎😍😘🥰😗😙😚☺🙂🤗🤩🤔🤨😐😑😶🙄😏😣😥😮🤐😯😪asdfghjk"\nexample3 = generateHDColour(THISDIR + "/test.png", 40)\n\n# Print\nprint(example)\nprint()\nprint(example2)\nprint()\nprint(example3)\nprint()\n\n# To SVG\nansiToSVG(example, THISDIR + "/example.svg")\nansiToSVG(example2, THISDIR + "/example2.svg")\nansiToSVG(example3, THISDIR + "/example3.svg")\n\n# To Raster\nansiToRaster(example, THISDIR + "/example.png")\nansiToRaster(example2, THISDIR + "/example2.png")\nansiToRaster(example3, THISDIR + "/example3.png")\n\n# To SVGRaster\nansiToSVGRaster(example, THISDIR + "/svgExample.png")\nansiToSVGRaster(example2, THISDIR + "/svgExample2.png")\nansiToSVGRaster(example3, THISDIR + "/svgExample3.png")\n\n# To HTML\nansiToHTML(example, THISDIR + "/example.html")\nansiToHTML(example2, THISDIR + "/example2.html")\nansiToHTML(example3, THISDIR + "/example3.html")\n\n# To HTMLRaster\nansiToHTMLRaster(example, THISDIR + "/htmlExample.png")\nansiToHTMLRaster(example2, THISDIR + "/htmlExample2.png")\nansiToHTMLRaster(example3, THISDIR + "/htmlExample3.png")\n```\n\n### SVG Image\n![example](test/example.svg)\n\n![example2](test/example2.svg)\n\n![example3](test/example3.svg)\n\n### Raster Image\n![example](test/example.png)\n\n![example2](test/example2.png)\n\n![example3](test/example3.png)\n\n### SVGRaster Image\n![example](test/svgExample.png)\n\n![example2](test/svgExample2.png)\n\n![example3](test/svgExample3.png)\n\n### HTML/ HTMLRaster Image\n![example](test/htmlExample.png)\n\n![example2](test/htmlExample2.png)\n\n![example3](test/htmlExample3.png)\n\n\n### Windows Terminal\n\n![winterm](readme-assets/terminal.png)\n\n## Choosing ansiToSVG, ansiToRaster, ansiToSVGRaster, ansiToHTML or ansiToHTMLRaster\n\n### ansiToSVG\nThis is better for the vast majority of cases as the image sizes are smaller\nfor reasonably simple ANSI sequences. The image size scales proportionally\nwith the length of the ANSI sequence. A large number of applications tend to\nopt for shorter sequences for output making `ansiToSVG` the better option.\n`ansiToSVG` also handles emoji as well as the OS does. For instance, on Windows\n10 one can expect full colour emoji. Image sizes can get out of hand for some\ncases such as catimage output as those tend to be very long ANSI sequences.\n\n### ansiToRaster\nThe image size does not scale to the length of the ANSI sequence but does scale\nto the number of lines of terminal output. This is ideal for output of complex\nANSI sequences that would be huge if `ansiToSVG` were used. However, emojis are\nin black and white and show quite poorly on coloured backgrounds.\n\n### ansiToSVGRaster\nTakes the advantages that `ansiToRaster` has whilst keeping colour emojis, Yay!\nThis uses pyppeteer to fire up a headless browser which opens the SVG and takes\na screenshot.\n\n### ansiToHTML\nHas the same advantages and disadvantages of `ansiToSVG` though this is not\nsuitable to be included in a GitHub readme\n\n### ansiToHTMLRaster\nHas the same advantages and disadvantages of `ansiToSVGRaster`\n\n\n## Changelog\nSee the [CHANGELOG](/CHANGELOG.md) for more information.\n\n## Install With PIP\n\n```python\npip install ansitoimg\n```\n\nHead to https://pypi.org/project/ansitoimg/ for more info\n\n## Language information\n### Built for\nThis program has been written for Python 3 and has been tested with\nPython version 3.8.0 <https://www.python.org/downloads/release/python-380/>.\n\n## Install Python on Windows\n### Chocolatey\n```powershell\nchoco install python\n```\n### Download\nTo install Python, go to <https://www.python.org/> and download the latest\nversion.\n\n## Install Python on Linux\n### Apt\n```bash\nsudo apt install python3.8\n```\n\n## How to run\n### With VSCode\n1. Open the .py file in vscode\n2. Ensure a python 3.8 interpreter is selected (Ctrl+Shift+P > Python:Select\nInterpreter > Python 3.8)\n3. Run by pressing Ctrl+F5 (if you are prompted to install any modules, accept)\n### From the Terminal\n```bash\n./[file].py\n```\n\n## How to update, build and publish\n\n1. Ensure you have installed the following dependencies\n\tLinux\n\t```bash\n\twget dephell.org/install | python3.8\n\twget https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python3.8\n\t```\n\tWindows\n\t```powershell\n\t(wget dephell.org/install -UseBasicParsing).Content | python\n\t(wget https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py -UseBasicParsing).Content | python\n\t```\n2. Use poetry for the heavy lifting and dephell to generate requirements\n\t```bash\n\tpoetry update\n\tdephell deps convert\n\t```\n3. Build/ Publish\n\t```bash\n\tpoetry build\n\tpoetry publish\n\t```\n\tor\n\t```bash\n\tpoetry publish --build\n\t```\n## Community Files\n### Licence\nMIT License\nCopyright (c) FredHappyface\n(See the [LICENSE](/LICENSE.md) for more information.)\n\n### Changelog\nSee the [Changelog](/CHANGELOG.md) for more information.\n\n### Code of Conduct\nIn the interest of fostering an open and welcoming environment, we\nas contributors and maintainers pledge to make participation in our\nproject and our community a harassment-free experience for everyone.\nPlease see the\n[Code of Conduct](https://github.com/FHPythonUtils/.github/blob/master/CODE_OF_CONDUCT.md) for more information.\n\n### Contributing\nContributions are welcome, please see the [Contributing Guidelines](https://github.com/FHPythonUtils/.github/blob/master/CONTRIBUTING.md) for more information.\n\n### Security\nThank you for improving the security of the project, please see the [Security Policy](https://github.com/FHPythonUtils/.github/blob/master/SECURITY.md) for more information.\n\n### Support\nThank you for using this project, I hope it is of use to you. Please be aware that\nthose involved with the project often do so for fun along with other commitments\n(such as work, family, etc). Please see the [Support Policy](https://github.com/FHPythonUtils/.github/blob/master/SUPPORT.md) for more information.\n',
    'author': 'FredHappyface',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/FHPythonUtils/AnsiToImg',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
